﻿;
; Module name   : Scroll
; Author        : mestnyi
; Last updated  : Aug 7, 2018
; Forum link    : https://www.purebasic.fr/english/posting.php?mode=edit&f=12&p=521603
; 

DeclareModule Scroll
  EnableExplicit
  #Vertical = #PB_ScrollBar_Vertical
  
  Structure PAGE
    Pos.l
    Length.l
  EndStructure
  
  Structure COLOR
    Front.l[4]
    Fore.l[4]
    Back.l[4]
    Line.l[4]
    Frame.l[4]
  EndStructure
  
  Structure STRUCT
    Max.l
    Min.l
    
    Type.l
    Focus.l
    Buttons.l
    Vertical.l
    ButtonLength.l
    
    Hide.b[2]
    Disable.b[2]
    DrawingMode.l
    
    y.l[4]
    x.l[4]
    Height.l[4]
    Width.l[4]
    
    Page.PAGE
    Area.PAGE
    Thumb.PAGE
    Color.Color[4]
  EndStructure
  
  Declare.b Draw(*Scroll.Struct)
  Declare.l Y(*Scroll.Struct)
  Declare.l X(*Scroll.Struct)
  Declare.l Width(*Scroll.Struct)
  Declare.l Height(*Scroll.Struct)
  Declare.b SetState(*Scroll.Struct, ScrollPos.l)
  Declare.l SetAttribute(*Scroll.Struct, Attribute.l, Value.l)
  Declare.b SetColor(*Scroll.Struct, ColorType.l, Color.l, Item.l=- 1, State.l=1)
  Declare.b Resize(*This.Struct, iX.l,iY.l,iWidth.l,iHeight.l, *Scroll.Struct=#Null)
  Declare.b Resizes(*v.Struct, *h.Struct, X,Y,Width,Height)
  Declare.b Updates(*v.Struct, *h.Struct, ScrollArea_X, ScrollArea_Y, ScrollArea_Width, ScrollArea_Height)
  Declare.b CallBack(*Scroll.Struct, EventType.l, MouseX.l, MouseY.l, WheelDelta.l=0)
  Declare.b Gadget(*Scroll.Struct, X.l,Y.l,Width.l,Height.l, Min.l, Max.l, PageLength.l, Flag.l)
EndDeclareModule

Module Scroll
  Macro Colors(_this_, _i_, _ii_, _iii_)
    If _this_\Color[_i_]\Line[_ii_]
      _this_\Color[_i_]\Line[_iii_] = _this_\Color[_i_]\Line[_ii_]
    Else
      _this_\Color[_i_]\Line[_iii_] = _this_\Color[0]\Line[_ii_]
    EndIf
    
    If _this_\Color[_i_]\Fore[_ii_]
      _this_\Color[_i_]\Fore[_iii_] = _this_\Color[_i_]\Fore[_ii_]
    Else
      _this_\Color[_i_]\Fore[_iii_] = _this_\Color[0]\Fore[_ii_]
    EndIf
    
    If _this_\Color[_i_]\Back[_ii_]
      _this_\Color[_i_]\Back[_iii_] = _this_\Color[_i_]\Back[_ii_]
    Else
      _this_\Color[_i_]\Back[_iii_] = _this_\Color[0]\Back[_ii_]
    EndIf
    
    If _this_\Color[_i_]\Frame[_ii_]
      _this_\Color[_i_]\Frame[_iii_] = _this_\Color[_i_]\Frame[_ii_]
    Else
      _this_\Color[_i_]\Frame[_iii_] = _this_\Color[0]\Frame[_ii_]
    EndIf
  EndMacro
  
  Macro BoxGradient(Type, X,Y,Width,Height,Color1,Color2)
    BackColor(Color1)
    FrontColor(Color2)
    If Type
      LinearGradient(X, Y, (X+Width), Y)
    Else
      LinearGradient(X, Y, X, (Y+Height))
    EndIf
    Box(X,Y,Width,Height)
  EndMacro
  
  Macro ResetColor(This)
    
    Colors(This, 0, 1, 0)
    Colors(This, 1, 1, 0)
    Colors(This, 2, 1, 0)
    Colors(This, 3, 1, 0)
    
    
    Colors(This, 1, 1, 1)
    Colors(This, 2, 1, 1)
    Colors(This, 3, 1, 1)
    
    Colors(This, 1, 2, 2)
    Colors(This, 2, 2, 2)
    Colors(This, 3, 2, 2)
    
    Colors(This, 1, 3, 3)
    Colors(This, 2, 3, 3)
    Colors(This, 3, 3, 3)
    
  EndMacro
  
  Procedure DrawArrow(X,Y, Size, Direction, Color, Thickness = 1) ; Рисуем стрелки
    Protected I
    
    ;ClipOutput(X-Thickness , Y-Thickness, Size*2+Thickness, Size*2+Thickness)
    
    If Direction = 1
      For i = 0 To Size 
        ; в верх
        LineXY((X+i)+Size,(Y+i-1)-(Thickness),(X+i)+Size,(Y+i-1)+(Thickness),Color) ; Левая линия
        LineXY(((X+(Size))-i),(Y+i-1)-(Thickness),((X+(Size))-i),(Y+i-1)+(Thickness),Color) ; правая линия
      Next
    ElseIf Direction = 3
      For i = 0 To Size
        ; в низ
        LineXY((X+i),(Y+i)-(Thickness),(X+i),(Y+i)+(Thickness),Color) ; Левая линия
        LineXY(((X+(Size*2))-i),(Y+i)-(Thickness),((X+(Size*2))-i),(Y+i)+(Thickness),Color) ; правая линия
      Next
    ElseIf Direction = 0 ; в лево
      For i = 0 To Size  
        ; в лево
        LineXY(((X+1)+i)-(Thickness),(((Y-2)+(Size))-i),((X+1)+i)+(Thickness),(((Y-2)+(Size))-i),Color) ; правая линия
        LineXY(((X+1)+i)-(Thickness),((Y-2)+i)+Size,((X+1)+i)+(Thickness),((Y-2)+i)+Size,Color)         ; Левая линия
      Next
    ElseIf Direction = 2 ; в право
      For i = 0 To Size
        ; в право
        LineXY(((X+2)+i)-(Thickness),((Y-2)+i),((X+2)+i)+(Thickness),((Y-2)+i),Color) ; Левая линия
        LineXY(((X+2)+i)-(Thickness),(((Y-2)+(Size*2))-i),((X+2)+i)+(Thickness),(((Y-2)+(Size*2))-i),Color) ; правая линия
      Next
    EndIf
    
    ;UnclipOutput()
  EndProcedure
  
  Procedure.l Pos(*This.Struct, ThumbPos.l)
    Protected ScrollPos.l
    
    With *This
      ScrollPos = \Min + Round((ThumbPos - \Area\Pos) / (\Area\Length / (\Max-\Min)), #PB_Round_Nearest)
      If (\Vertical And \Type = #PB_GadgetType_TrackBar) : ScrollPos = ((\Max-\Min)-ScrollPos) : EndIf
    EndWith
    
    ProcedureReturn ScrollPos
  EndProcedure
  
  Procedure.l ThumbPos(*This.Struct, ScrollPos.l)
    Protected ThumbPos.l
    
    With *This
      ThumbPos = (\Area\Pos + Round((ScrollPos-\Min) * (\Area\Length / (\Max-\Min)), #PB_Round_Nearest))
    EndWith
    
    ProcedureReturn ThumbPos
  EndProcedure
  
  Procedure.l ThumbLength(*This.Struct)
    Protected ThumbLength.l
    
    With *This
      ThumbLength = Round(\Area\Length - (\Area\Length / (\Max-\Min))*((\Max-\Min) - \Page\Length), #PB_Round_Nearest)
    EndWith
    
    ProcedureReturn ThumbLength
  EndProcedure
  
  ;-
  Procedure.b Draw(*Scroll.Struct)
    With *Scroll
      If Not \Hide
        DrawingMode(#PB_2DDrawing_Default)
        
        If \Vertical
          ; Если есть вертикальная и горизонтальная полоса,
          ; то окрашиваем прамоугольник между ними
          If \Height>(\Y[2]+\Height[2])
            Line(\X[0],\Y[0]+1,1,\Y[2]+\Height[2]-2,\Color[0]\Line)
            Box(\X[0]+1,\Y[0],\Width[0]-1,\Height[0]-\Y[0],\Color[0]\Back)
            Line(\X[0],\Y[2]+\Height[2],\Width[0],1,\Color[0]\Line)
          Else
            Line(\X[0],\Y[0]+1,1,\Height[0]-2,\Color[0]\Line)
            Box(\X[0]+1,\Y[0],\Width[0]-1,\Height[0],\Color[0]\Back)
          EndIf
        Else
          ; Если есть вертикальная и горизонтальная полоса,
          ; то окрашиваем прамоугольник между ними
          If \Width>(\X[2]+\Width[2])
            Line(\X[0]+1,\Y[0],\X[2]+\Width[2]-2,1,\Color[0]\Line)
            Box(\X[0],\Y[0]+1,\Width[0]-\X[0],\Height[0]-1,\Color[0]\Back)
            Line(\X[2]+\Width[2],\Y[0],1,\Height[0],\Color[0]\Line)
          Else
            Line(\X[0]+1,\Y[0],\Width[0]-2,1,\Color[0]\Line)
            Box(\X[0],\Y[0]+1,\Width[0],\Height[0]-1,\Color[0]\Back)
          EndIf
        EndIf
        
        ;Case #PB_2DDrawing_Gradient
        DrawingMode(\DrawingMode)
        BoxGradient(\Vertical,\X[3],\Y[3],\Width[3],\Height[3],\Color[3]\Fore,\Color[3]\Back)
        
        BackColor(#PB_Default)
        FrontColor(#PB_Default) ; bug
        
        If \DrawingMode = #PB_2DDrawing_Gradient
          DrawingMode(#PB_2DDrawing_Outlined)
          Box(\X[3],\Y[3],\Width[3],\Height[3],\Color[3]\Frame)
        EndIf
        
        If \ButtonLength
          DrawingMode(\DrawingMode)
          BoxGradient(\Vertical,\X[1],\Y[1],\Width[1],\Height[1],\Color[1]\Fore,\Color[1]\Back)
          BoxGradient(\Vertical,\X[2],\Y[2],\Width[2],\Height[2],\Color[2]\Fore,\Color[2]\Back)
          
          If \DrawingMode = #PB_2DDrawing_Gradient
            DrawingMode(#PB_2DDrawing_Outlined)
            Box(\X[1],\Y[1],\Width[1],\Height[1],\Color[1]\Frame)
            Box(\X[2],\Y[2],\Width[2],\Height[2],\Color[2]\Frame)
          EndIf
          
          DrawingMode(#PB_2DDrawing_Default)
          DrawArrow(\X[1]+(\Width[1]-6)/2,\Y[1]+(\Height[1]-3)/2, 3, Bool(\Vertical), \Color[1]\Line)
          DrawArrow(\X[2]+(\Width[2]-6)/2,\Y[2]+(\Height[2]-3)/2, 3, Bool(\Vertical)+2, \Color[2]\Line)
        EndIf
        
        If \DrawingMode = #PB_2DDrawing_Gradient
          DrawingMode(#PB_2DDrawing_Default)
          If \Vertical
            Line(\X[3]+(\Width[3]-10)/2,\Y[3]+\Height[3]/2-3,10,1,\Color[3]\Line)
            Line(\X[3]+(\Width[3]-10)/2,\Y[3]+\Height[3]/2,10,1,\Color[3]\Line)
            Line(\X[3]+(\Width[3]-10)/2,\Y[3]+\Height[3]/2+3,10,1,\Color[3]\Line)
          Else
            Line(\X[3]+\Width[3]/2-3,\Y[3]+(\Height[3]-10)/2,1,10,\Color[3]\Line)
            Line(\X[3]+\Width[3]/2,\Y[3]+(\Height[3]-10)/2,1,10,\Color[3]\Line)
            Line(\X[3]+\Width[3]/2+3,\Y[3]+(\Height[3]-10)/2,1,10,\Color[3]\Line)
          EndIf
        EndIf
      EndIf
    EndWith 
    
  EndProcedure
  
  Procedure.l X(*Scroll.Struct)
    Protected Result.l
    
    If *Scroll
      With *Scroll
        If \Hide
          Result = \X+\Width
        Else
          Result = \X
        EndIf
      EndWith
    EndIf
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.l Y(*Scroll.Struct)
    Protected Result.l
    
    If *Scroll
      With *Scroll
        If \Hide 
          Result = \Y+\Height
        Else
          Result = \Y
        EndIf
      EndWith
    EndIf
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.l Width(*Scroll.Struct)
    Protected Result.l
    
    If *Scroll
      With *Scroll
        If Not \Hide And \Width
          Result = \Width
        EndIf
      EndWith
    EndIf
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.l Height(*Scroll.Struct)
    Protected Result.l
    
    If *Scroll
      With *Scroll
        If Not \Hide And \Height
          Result = \Height
        EndIf
      EndWith
    EndIf
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.b SetState(*Scroll.Struct, ScrollPos.l)
    Protected Result.b
    
    With *Scroll
      If (\Vertical And \Type = #PB_GadgetType_TrackBar) : ScrollPos = ((\Max-\Min)-ScrollPos) : EndIf
      
      If ScrollPos < \Min : ScrollPos = \Min : EndIf
      If ScrollPos > (\Max-\Page\Length)
        ScrollPos = (\Max-\Page\Length)
      EndIf
      
      If \Page\Pos<>ScrollPos : \Page\Pos=ScrollPos
        \Thumb\Pos = ThumbPos(*Scroll, ScrollPos)
        
        If \Vertical
          \Y[3] = \Thumb\Pos
          \Height[3] = \Thumb\Length
        Else
          \X[3] = \Thumb\Pos
          \Width[3] = \Thumb\Length
        EndIf
        
        Result = #True
      EndIf
    EndWith
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.l SetAttribute(*Scroll.Struct, Attribute.l, Value.l)
    Protected Result.l
    
    With *Scroll
      Select Attribute
        Case #PB_ScrollBar_Minimum
          If \Min <> Value
            \Min = Value
            \Page\Pos = Value
            Result = #True
          EndIf
          
        Case #PB_ScrollBar_Maximum
          If \Max <> Value
            If \Min > Value
              \Max = \Min + 1
            Else
              \Max = Value
            EndIf
            Result = #True
          EndIf
          
        Case #PB_ScrollBar_PageLength
          If \Page\Length <> Value
            If Value > (\Max-\Min) : \Max = Value ; Если этого page_length вызвать раньше maximum то не правильно работает
              \Page\Length = (\Max-\Min)
            Else
              \Page\Length = Value
            EndIf
            
            Result = #True
          EndIf
          
      EndSelect
    EndWith
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.b SetColor(*Scroll.Struct, ColorType.l, Color.l, Item.l=- 1, State.l=1)
    Protected Result
    
    With *Scroll
      Select ColorType
        Case #PB_Gadget_LineColor
          If Item=- 1
            \Color\Line[State] = Color
          Else
            \Color[Item]\Line[State] = Color
          EndIf
          
        Case #PB_Gadget_BackColor
          If Item=- 1
            \Color\Back[State] = Color
          Else
            \Color[Item]\Back[State] = Color
          EndIf
          
        Case #PB_Gadget_FrontColor
        Default ; Case #PB_Gadget_FrameColor
          If Item=- 1
            \Color\Frame[State] = Color
          Else
            \Color[Item]\Frame[State] = Color
          EndIf
          
      EndSelect
    EndWith
    
    ResetColor(*Scroll)
    
    ProcedureReturn Bool(Color)
  EndProcedure
  
  Procedure.b Resize(*This.Struct, X.l,Y.l,Width.l,Height.l, *Scroll.Struct=#Null)
    Protected Result, Lines
    
    With *This
      Lines = Bool(\Type=#PB_GadgetType_ScrollBar)
      
      If *Scroll
        If \Vertical
          If Height=#PB_Ignore
            If *Scroll\Hide
              Height = (*Scroll\Y + *Scroll\Height) - \Y
            Else
              Height = *Scroll\Y - \Y;*2
            EndIf
          EndIf
        Else
          If Width=#PB_Ignore
            If *Scroll\Hide
              Width = (*Scroll\X + *Scroll\Width) - \X
            Else
              Width = *Scroll\X - \X;*2
            EndIf
          EndIf
        EndIf
      EndIf
      
      ;
      If X=#PB_Ignore : X = \X[0] : EndIf
      If Y=#PB_Ignore : Y = \Y[0] : EndIf
      If Width=#PB_Ignore : Width = \Width[0] : EndIf
      If Height=#PB_Ignore : Height = \Height[0] : EndIf
      
      Protected ScrollPage = ((\Max-\Min) - \Page\Length)
      ;
      If ((\Max-\Min) >= \Page\Length)
        If \Vertical
          \Area\Pos = Y+\ButtonLength
          \Area\Length = (Height-\ButtonLength*2)
        Else
          \Area\Pos = X+\ButtonLength
          \Area\Length = (Width-\ButtonLength*2)
        EndIf
        
        If \Area\Length
          \Thumb\Length = ThumbLength(*This)
          
          If (\Area\Length > \ButtonLength)
            If \ButtonLength
              If (\Thumb\Length < \ButtonLength)
                \Area\Length = Round(\Area\Length - (\ButtonLength-\Thumb\Length), #PB_Round_Nearest)
                \Thumb\Length = \ButtonLength 
              EndIf
            Else
              If (\Thumb\Length < 7)
                \Area\Length = Round(\Area\Length - (7-\Thumb\Length), #PB_Round_Nearest)
                \Thumb\Length = 7
              EndIf
            EndIf
          Else
            \Thumb\Length = \Area\Length 
          EndIf
          
          If \Area\Length > 0
            If (\Type <> #PB_GadgetType_TrackBar) And (\Thumb\Pos+\Thumb\Length) >= (\Area\Length+\ButtonLength)
              SetState(*This, ScrollPage)
            EndIf
            
            \Thumb\Pos = ThumbPos(*This, \Page\Pos)
          EndIf
        EndIf
      EndIf
      
      ; скролл баре
      \X[0] = X : \Y[0] = Y : \Width[0] = Width : \Height[0] = Height 
      
      If \Vertical
        ; Верхняя кнопка на скролл баре
        \X[1] = X + Lines : \Y[1] = Y : \Width[1] = Width - Lines : \Height[1] = \ButtonLength
        
        ; Нижняя кнопка на скролл баре
        \X[2] = X + Lines : \Width[2] = Width - Lines : \Height[2] = \ButtonLength : \Y[2] = Y+Height-\Height[2]
        
        ; Ползунок на скролл баре
        \X[3] = X + Lines : \Width[3] = Width - Lines : \Y[3] = \Thumb\Pos : \Height[3] = \Thumb\Length
        
      Else
        ; Верхняя кнопка на скролл баре
        \X[1] = X : \Y[1] = Y + Lines : \Width[1] = \ButtonLength : \Height[1] = Height - Lines
        
        ; Нижняя кнопка на скролл баре
        \Y[2] = Y + Lines : \Height[2] = Height - Lines : \Width[2] = \ButtonLength : \X[2] = X+Width-\Width[2]
        
        ; Ползунок на скролл баре
        \Y[3] = Y + Lines : \Height[3] = Height - Lines : \X[3] = \Thumb\Pos : \Width[3] = \Thumb\Length
        
      EndIf
      
      ProcedureReturn Bool(Not ((\Max-\Min) > \Page\Length))
    EndWith
  EndProcedure
  
  Procedure.b Updates(*v.Struct, *h.Struct, ScrollArea_X, ScrollArea_Y, ScrollArea_Width, ScrollArea_Height)
    Protected iWidth = X(*v), iHeight = Y(*h)
    Static hPos, vPos : vPos = *v\Page\Pos : hPos = *h\Page\Pos
    
    ; Вправо работает как надо
    If ScrollArea_Width<*h\Page\Pos+iWidth 
      ScrollArea_Width=*h\Page\Pos+iWidth
      ; Влево работает как надо
    ElseIf ScrollArea_X>*h\Page\Pos And
           ScrollArea_Width=*h\Page\Pos+iWidth 
      ScrollArea_Width = iWidth 
    EndIf
    
    ; Вниз работает как надо
    If ScrollArea_Height<*v\Page\Pos+iHeight
      ScrollArea_Height=*v\Page\Pos+iHeight 
      ; Верх работает как надо
    ElseIf ScrollArea_Y>*v\Page\Pos And
           ScrollArea_Height=*v\Page\Pos+iHeight 
      ScrollArea_Height = iHeight 
    EndIf
    
    If ScrollArea_X>0 : ScrollArea_X=0 : EndIf
    If ScrollArea_Y>0 : ScrollArea_Y=0 : EndIf
    
    If ScrollArea_X<*h\Page\Pos : ScrollArea_Width-ScrollArea_X : EndIf
    If ScrollArea_Y<*v\Page\Pos : ScrollArea_Height-ScrollArea_Y : EndIf
    
    If *v\Max<>ScrollArea_Height : SetAttribute(*v, #PB_ScrollBar_Maximum, ScrollArea_Height) : EndIf
    If *h\Max<>ScrollArea_Width : SetAttribute(*h, #PB_ScrollBar_Maximum, ScrollArea_Width) : EndIf
    
    If *v\Page\Length<>iHeight : SetAttribute(*v, #PB_ScrollBar_PageLength, iHeight) : EndIf
    If *h\Page\Length<>iWidth : SetAttribute(*h, #PB_ScrollBar_PageLength, iWidth) : EndIf
    
    If ScrollArea_Y<0 : SetState(*v, (ScrollArea_Height-ScrollArea_Y)-ScrollArea_Height) : EndIf
    If ScrollArea_X<0 : SetState(*h, (ScrollArea_Width-ScrollArea_X)-ScrollArea_Width) : EndIf
    
    *v\Hide = Resize(*v, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore, *h) 
    *h\Hide = Resize(*h, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore, *v)
    
    If *v\Hide : *v\Page\Pos = 0 : If vPos : *v\Hide = vPos : EndIf : Else : *v\Page\Pos = vPos : *h\Width = iWidth+*v\Width : EndIf
    If *h\Hide : *h\Page\Pos = 0 : If hPos : *h\Hide = hPos : EndIf : Else : *h\Page\Pos = hPos : *v\Height = iHeight+*h\Height : EndIf
    
    ProcedureReturn Bool(ScrollArea_Height>=iHeight Or ScrollArea_Width>=iWidth)
  EndProcedure
  
  Procedure.b Resizes(*v.Struct, *h.Struct, X,Y,Width,Height )
; ;     Static.l w,h
;     
; ;     If Width=#PB_Ignore 
; ;       Width = *v\X+*v\Width
; ;       If Not *v\Hide And w 
; ;         Width+w : w=0 
; ;       EndIf
; ;     Else
; ;       Width+*h\x 
; ;       w=X
; ;     EndIf
; ;     If Height=#PB_Ignore 
; ;       Height = *h\Y+*h\Height
; ;       If Not *h\Hide And h
; ;         Height+h : h=0
; ;       EndIf
; ;     Else
; ;       Height+*v\y 
; ;       h=Y
; ;     EndIf
    
    If Width=#PB_Ignore 
      Width = *v\X+*v\Width
    Else
      Width+x 
    EndIf
    If Height=#PB_Ignore 
      Height = *h\Y+*h\Height
    Else
      Height+y 
    EndIf
    
    Protected iWidth = Width-Width(*v), iHeight = Height-Height(*h)
    
    If *v\Page\Length<>iHeight : SetAttribute(*v, #PB_ScrollBar_PageLength, iHeight) : EndIf
    If *h\Page\Length<>iWidth : SetAttribute(*h, #PB_ScrollBar_PageLength, iWidth) : EndIf
    
    *v\Hide = Resize(*v, Width-*v\Width, Y, #PB_Ignore, #PB_Ignore, *h) : iWidth = Width-Width(*v)
    *h\Hide = Resize(*h, X, Height-*h\Height, #PB_Ignore, #PB_Ignore, *v) : iHeight = Height-Height(*h)
    
    If *v\Page\Length<>iHeight : SetAttribute(*v, #PB_ScrollBar_PageLength, iHeight) : EndIf
    If *h\Page\Length<>iWidth : SetAttribute(*h, #PB_ScrollBar_PageLength, iWidth) : EndIf
    
    *v\Hide = Resize(*v, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore, *h)
    *h\Hide = Resize(*h, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore, *v)
    
    If *v\Hide 
      *v\Page\Pos = 0 
    Else
      ; Если есть вертикальная и горизонтальная полоса,
      ; то окрашиваем прамоугольник между ними
      *h\Width = Width
    EndIf
    If *h\Hide 
      *h\Page\Pos = 0 
    Else
      ; Если есть вертикальная и горизонтальная полоса,
      ; то окрашиваем прамоугольник между ними
      *v\Height = Height
    EndIf
    
    ProcedureReturn Bool(*v\Hide|*h\Hide)
  EndProcedure
  
  Procedure.b CallBack(*Scroll.Struct, EventType.l, MouseX.l, MouseY.l, WheelDelta.l=0)
    Protected Result, Buttons
    Static LastX, LastY, Last, *This.Struct, Cursor, Drag
    
    With *Scroll
      If \Hide
        If *Scroll = *This
          \Focus = 0
          \Buttons = 0
        EndIf
      Else
        If GetGadgetAttribute(EventGadget(), #PB_Canvas_Buttons) & #PB_Canvas_LeftButton ; Drag
          If \Buttons>0 : Buttons = \Buttons : EndIf
        Else
          If (Mousex>=\x And Mousex<\x+\Width And Mousey>\y And Mousey=<\y+\Height) 
            If (Mousex>\x[1] And Mousex=<\x[1]+\Width[1] And  Mousey>\y[1] And Mousey=<\y[1]+\Height[1])
              \Buttons = 1
            ElseIf (Mousex>\x[3] And Mousex=<\x[3]+\Width[3] And Mousey>\y[3] And Mousey=<\y[3]+\Height[3])
              \Buttons = 3
            ElseIf (Mousex>\x[2] And Mousex=<\x[2]+\Width[2] And Mousey>\y[2] And Mousey=<\y[2]+\Height[2])
              \Buttons = 2
            Else
              \Buttons =- 1
            EndIf
            
            If \Buttons>0 : Buttons = \Buttons : EndIf
          ElseIf Not Drag
            \Buttons = 0
          EndIf
        EndIf
        Select EventType
          Case #PB_EventType_MouseWheel  
            If *This = *Scroll
              Select WheelDelta
                Case-1 : Result = SetState(*Scroll, \Page\Pos - (\Max-\Min)/30)
                Case 1 : Result = SetState(*Scroll, \Page\Pos + (\Max-\Min)/30)
              EndSelect
            EndIf
            
          Case #PB_EventType_MouseLeave : Buttons = 0 : LastX = 0 : LastY = 0
          Case #PB_EventType_LeftButtonUp : Drag = 0 :  LastX = 0 : LastY = 0
          Case #PB_EventType_LeftButtonDown : Drag = 1
            If Buttons : *This = *Scroll : EndIf
            Select Buttons
              Case 0
                If *This = *Scroll Or (\Height>(\Y[2]+\Height[2]) And \Buttons =- 1) 
                  If \Vertical
                    Result = SetState(*Scroll, Pos(*Scroll, (MouseY-\Thumb\Length/2)))
                  Else
                    Result = SetState(*Scroll, Pos(*Scroll, (MouseX-\Thumb\Length/2)))
                  EndIf
                EndIf
              Case 1 : Result = SetState(*This, \Page\Pos - \Max/100)
              Case 2 : Result = SetState(*This, \Page\Pos + \Max/100)
              Case 3 : LastX = MouseX - \Thumb\Pos : LastY = MouseY - \Thumb\Pos
            EndSelect
            
          Case #PB_EventType_MouseMove
            If Drag
              If Bool(LastX|LastY) 
                If *This = *Scroll
                  If \Vertical
                    Result = SetState(*Scroll, Pos(*Scroll, (MouseY-LastY)))
                  Else
                    Result = SetState(*Scroll, Pos(*Scroll, (MouseX-LastX)))
                  EndIf
                EndIf
              EndIf
            Else
              If \Buttons
                If Last<>Buttons
                  If *This : CallBack(*This, #PB_EventType_MouseLeave, MouseX, MouseY, WheelDelta) : EndIf
                  EventType = #PB_EventType_MouseEnter
                  Last = Buttons
                EndIf
                
                If *This <> *Scroll 
                  ; Debug "Мышь находится внутри"
                  Cursor = GetGadgetAttribute(EventGadget(), #PB_Canvas_Cursor)
                  SetGadgetAttribute(EventGadget(), #PB_Canvas_Cursor, #PB_Cursor_Default)
                  *This = *Scroll
                EndIf
              ElseIf *This = *Scroll
                If Cursor <> GetGadgetAttribute(EventGadget(), #PB_Canvas_Cursor)
                  ; Debug "Мышь находится снаружи"
                  SetGadgetAttribute(EventGadget(), #PB_Canvas_Cursor, Cursor)
                EndIf
                
                EventType = #PB_EventType_MouseLeave
                *This = 0
                Last = 0
              EndIf
            EndIf
            
        EndSelect
        
        Select EventType
          Case #PB_EventType_LeftButtonDown, #PB_EventType_LeftButtonUp, #PB_EventType_MouseEnter, #PB_EventType_MouseLeave
            If Buttons
              \Color[Buttons]\Fore = \Color[Buttons]\Fore[2+Bool(EventType=#PB_EventType_LeftButtonDown)]
              \Color[Buttons]\Back = \Color[Buttons]\Back[2+Bool(EventType=#PB_EventType_LeftButtonDown)]
              \Color[Buttons]\Frame = \Color[Buttons]\Frame[2+Bool(EventType=#PB_EventType_LeftButtonDown)]
              \Color[Buttons]\Line = \Color[Buttons]\Line[2+Bool(EventType=#PB_EventType_LeftButtonDown)]
            Else
              ResetColor(*Scroll)
            EndIf
            
            Result = #True
        EndSelect 
        
        Select EventType
          Case #PB_EventType_Focus : \Focus = #True : Result = #True
          Case #PB_EventType_LostFocus : \Focus = #False : Result = #True
        EndSelect
      EndIf
    EndWith
    
    ProcedureReturn Result
  EndProcedure
  
  Procedure.b Gadget(*Scroll.Struct, X.l,Y.l,Width.l,Height.l, Min.l, Max.l, PageLength.l, Flag.l)
    
    With *Scroll
      \Color[0]\Fore[1] = $F6F6F6 
      \Color[0]\Frame[1] = $BABABA
      
      \Color[0]\Back[1] = $F0F0F0 
      \Color[1]\Back[1] = $E2E2E2  
      \Color[2]\Back[1] = $E2E2E2 
      \Color[3]\Back[1] = $E2E2E2 
      
      \Color[0]\Line[1] = $FFFFFF
      \Color[1]\Line[1] = $5B5B5B
      \Color[2]\Line[1] = $5B5B5B
      \Color[3]\Line[1] = $5B5B5B
      
      ;
      \Color[0]\Fore[2] = $EAEAEA
      \Color[0]\Back[2] = $CECECE
      \Color[0]\Line[2] = $5B5B5B
      \Color[0]\Frame[2] = $8F8F8F
      
      ;
      \Color[0]\Fore[3] = $E2E2E2
      \Color[0]\Back[3] = $B4B4B4
      \Color[0]\Line[3] = $FFFFFF
      \Color[0]\Frame[3] = $6F6F6F
      
      ResetColor(*Scroll)
      
      \ButtonLength = 17
      \Type = #PB_GadgetType_ScrollBar
      
      \DrawingMode = #PB_2DDrawing_Gradient
      \Vertical = Bool(Flag&#PB_ScrollBar_Vertical)
      
      If \Min <> Min : SetAttribute(*Scroll, #PB_ScrollBar_Minimum, Min) : EndIf
      If \Max <> Max : SetAttribute(*Scroll, #PB_ScrollBar_Maximum, Max) : EndIf
      If \Page\Length <> Pagelength : SetAttribute(*Scroll, #PB_ScrollBar_PageLength, Pagelength) : EndIf
    EndWith
    
    ProcedureReturn Resize(*Scroll, X,Y,Width,Height)
  EndProcedure
  
EndModule

;- EXAMPLE
CompilerIf #PB_Compiler_IsMainFile
  If LoadImage(0, #PB_Compiler_Home+"Examples\Sources\Data\File.bmp")     ; Измените путь/имя файла на собственное изображение 32x32 пикселя
  EndIf
  Define a,i
  
  Global *Vertical.Scroll::Struct=AllocateStructure(Scroll::Struct)
  Global *Horisontal.Scroll::Struct=AllocateStructure(Scroll::Struct)
  
  Procedure CallBack()
    Protected Canvas = EventGadget()
    Protected MouseX = GetGadgetAttribute(Canvas, #PB_Canvas_MouseX)
    Protected MouseY = GetGadgetAttribute(Canvas, #PB_Canvas_MouseY)
    Protected WheelDelta = GetGadgetAttribute(EventGadget(), #PB_Canvas_WheelDelta)
    Protected Width = GadgetWidth(Canvas)
    Protected Height = GadgetHeight(Canvas)
    Protected iHeight = Height-Scroll::Height(*Horisontal)
    Protected iWidth = Width-Scroll::Width(*Vertical)
    
    Select EventType()
      Case #PB_EventType_Resize : ResizeGadget(Canvas, #PB_Ignore, #PB_Ignore, #PB_Ignore, #PB_Ignore) ; Bug (562)
                                                                                                       ; Scroll::SetAttribute(*Vertical, #PB_ScrollBar_PageLength, Height-20)
                                                                                                       ; Scroll::SetAttribute(*Horisontal, #PB_ScrollBar_PageLength, Width-55)
        
        
        Scroll::Resize(*Vertical, Width-35, #PB_Ignore, #PB_Ignore, Height-20)
        Scroll::Resize(*Horisontal, #PB_Ignore, #PB_Ignore, Width-55, #PB_Ignore)
        
        If StartDrawing(CanvasOutput(Canvas))
          Box(0,0,Width,Height)
          Scroll::Draw(*Vertical)
          Scroll::Draw(*Horisontal)
          StopDrawing()
        EndIf
        
    EndSelect
    
    If Scroll::CallBack(*Vertical, EventType(), MouseX, MouseY, WheelDelta) 
      If StartDrawing(CanvasOutput(Canvas))
        Scroll::Draw(*Vertical)
        StopDrawing()
      EndIf
    EndIf
    
    If Scroll::CallBack(*Horisontal, EventType(), MouseX, MouseY, WheelDelta)
      If StartDrawing(CanvasOutput(Canvas))
        Scroll::Draw(*Horisontal)
        StopDrawing()
      EndIf
    EndIf
    
  EndProcedure
  
  Procedure ResizeCallBack()
    ResizeGadget(1, #PB_Ignore, #PB_Ignore, WindowWidth(EventWindow(), #PB_Window_InnerCoordinate)-20, WindowHeight(EventWindow(), #PB_Window_InnerCoordinate)-20)
  EndProcedure
  
  If OpenWindow(0, 0, 0, 325, 160, "Scroll on the canvas", #PB_Window_SystemMenu | #PB_Window_SizeGadget | #PB_Window_ScreenCentered)
    CanvasGadget(1,  10,10,305,140, #PB_Canvas_Keyboard)
    SetGadgetAttribute(1, #PB_Canvas_Cursor, #PB_Cursor_Cross)
    
    With *Vertical
      Scroll::Gadget(*Vertical, 270, 10,  25, 120 ,0, 300, 50, #PB_ScrollBar_Vertical)
      Scroll::SetState(*Vertical, 100) 
    EndWith
    
    With *Horisontal
      *Horisontal\Hide = Scroll::Gadget(*Horisontal, 10, 42, 250,  20, 30, 100, 30, 0)
      *Horisontal\ButtonLength=0
      Scroll::SetColor(*Horisontal, #PB_Gadget_BackColor, $5A98FF)
      Scroll::SetState(*Horisontal, 50) 
    EndWith
    
    ;     If StartDrawing(CanvasOutput(1))
    ;       Scroll::Draw(*Vertical)
    ;       Scroll::Draw(*Horisontal)
    ;       StopDrawing()
    ;     EndIf
    
    PostEvent(#PB_Event_Gadget, 0,1,#PB_EventType_Resize)
    BindGadgetEvent(1, @CallBack())
    
    BindEvent(#PB_Event_SizeWindow, @ResizeCallBack())
    Repeat : Until WaitWindowEvent() = #PB_Event_CloseWindow
  EndIf
CompilerEndIf
; IDE Options = PureBasic 5.70 LTS beta 1 (Windows - x86)
; CursorPosition = 862
; FirstLine = 851
; Folding = -----
; EnableXP