﻿; +--------------+----------+
; | RatingGadget | by kenmo |
; +--------------+----------+
; |  8.01.2013 . Creation (PB 5.11)
; |   .02.     . Fixed Hover below 1 bug
; | 10.15.     . Embedded star icons in code
; |   .16.     . Cleaned up demo code
; |   .26.     . Minor optimization to SetRatingGadget()

;-

DeclareModule RatingGadget
EnableExplicit




;- Constants - Public

; Attribute constants
Enumeration
  #RG_OnImageID
  #RG_OffImageID
  #RG_Background
  #RG_State
  #RG_AllowZero
  #RG_Maximum
EndEnumeration



Declare ResizeRatingGadget(Gadget.i, x.i, y.i, Width.i, Height.i)
Declare.i FreeRatingGadget(Gadget.i)
Declare SetRatingGadget(Gadget.i, Attribute.i, Value.i)
Declare.i GetRatingGadget(Gadget.i, Attribute.i)
Declare.i RatingGadgetEvent(Gadget.i, Type.i)
Declare.i RatingGadget(Gadget.i, x.i, y.i, Width.i, Height.i, Max.i, OnImageID.i, OffImageID.i = #Null)

EndDeclareModule



Module RatingGadget
;- Structures - Private

Structure RATINGGADGET
  State.i
  Max.i
  Hover.i
  OnImageID.i
  OffImageID.i
  Background.i
  AllowZero.i
EndStructure
;-
;- Procedures - Private

Procedure.i xRatingGadgetDefault(Attribute.i)
  CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
    Select (Attribute)
      Case #RG_Background
        ProcedureReturn (GetSysColor_(#COLOR_BTNFACE))
    EndSelect
  CompilerElse
    Select (Attribute)
      Case #RG_Background
        ProcedureReturn ($EAEAEA)
    EndSelect
  CompilerEndIf
EndProcedure

Procedure xRatingGadgetRedraw(Gadget.i, *RG.RATINGGADGET, Empty.i = #False)
  Protected i.i, w.i, h.i
  
  If (IsGadget(Gadget) And *RG)
    If (*RG\Max < 1)
      *RG\Max = 1
    EndIf
    If (*RG\State > *RG\Max)
      *RG\State = *RG\Max
    Else
      If (*RG\AllowZero And (*RG\State < 0))
        *RG\State = 0
      ElseIf ((Not *RG\AllowZero) And (*RG\State < 1))
        *RG\State = 1
      EndIf
    EndIf
    
    If (StartDrawing(CanvasOutput(Gadget)))
      w = OutputWidth()
      h = OutputHeight()
      Box(0, 0, w, h, *RG\Background)
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      
      If (*RG\OnImageID)
        For i = 0 To *RG\Max - 1
          If ((Not Empty) And ((*RG\Hover And (i < *RG\Hover)) Or ((Not *RG\Hover) And (i < *RG\State))))
            DrawImage(*RG\OnImageID, i * w / *RG\Max, 0)
          ElseIf (*RG\OffImageID)
            DrawImage(*RG\OffImageID, i * w / *RG\Max, 0)
          EndIf
        Next i
      EndIf
      
      StopDrawing()
    EndIf
  EndIf
EndProcedure





;-
;- Procedures - Public

Procedure ResizeRatingGadget(Gadget.i, x.i, y.i, Width.i, Height.i)
  ResizeGadget(Gadget, x, y, Width, Height)
  xRatingGadgetRedraw(Gadget, GetGadgetData(Gadget))
EndProcedure

Procedure.i FreeRatingGadget(Gadget.i)
  Protected *RG.RATINGGADGET
  
  If (IsGadget(Gadget))
    *RG = GetGadgetData(Gadget)
    If (*RG)
      ClearStructure(*RG, RATINGGADGET)
      FreeMemory(*RG)
    EndIf
    FreeGadget(Gadget)
  EndIf
  
  ProcedureReturn (#Null)
EndProcedure

Procedure SetRatingGadget(Gadget.i, Attribute.i, Value.i)
  Protected *RG.RATINGGADGET
  
  If (IsGadget(Gadget))
    *RG = GetGadgetData(Gadget)
    If (*RG)
      Select (Attribute)
        Case #RG_OnImageID
          *RG\OnImageID = Value
        Case #RG_OffImageID
          *RG\OffImageID = Value
        Case #RG_Background
          *RG\Background = Value
        Case #RG_State
          If (Value < 0)
            Value = 0
          EndIf
          If (Value > *RG\Max)
            Value = *RG\Max
          EndIf
          *RG\State = Value
        Case #RG_AllowZero
          *RG\AllowZero = Bool(Value)
        Case #RG_Maximum
          *RG\Max = Value
      EndSelect
      xRatingGadgetRedraw(Gadget, *RG)
    EndIf
  EndIf
EndProcedure

Procedure.i GetRatingGadget(Gadget.i, Attribute.i)
  Protected Result = #Null
  Protected *RG.RATINGGADGET
  
  If (IsGadget(Gadget))
    *RG = GetGadgetData(Gadget)
    If (*RG)
      Select (Attribute)
        Case #RG_OnImageID
          Result = *RG\OnImageID
        Case #RG_OffImageID
          Result = *RG\OffImageID
        Case #RG_Background
          Result = *RG\Background
        Case #RG_State
          Result = *RG\State
        Case #RG_AllowZero
          Result = Bool(*RG\AllowZero)
        Case #RG_Maximum
          Result = *RG\Max
      EndSelect
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

Procedure.i RatingGadgetEvent(Gadget.i, Type.i)
  Protected Result.i = #Null
  Protected *RG.RATINGGADGET
  Protected Hover.i
  
  If (IsGadget(Gadget))
    *RG = GetGadgetData(Gadget)
    If (*RG)
      Hover = *RG\Hover
      Select (Type)
        Case #PB_EventType_LeftClick
          If (*RG\Hover)
            *RG\State = *RG\Hover
            Result = #PB_EventType_Change
            xRatingGadgetRedraw(Gadget, *RG)
          EndIf
        Case #PB_EventType_RightClick
          If (*RG\AllowZero)
            *RG\State = 0
            xRatingGadgetRedraw(Gadget, *RG, #True)
            Result = #PB_EventType_Change
          EndIf
        Case #PB_EventType_MouseEnter, #PB_EventType_MouseMove
          Hover = 1 + *RG\Max * GetGadgetAttribute(Gadget, #PB_Canvas_MouseX) / GadgetWidth(Gadget)
          If (Hover < 1)
            Hover = 1
          ElseIf (Hover > *RG\Max)
            Hover = *RG\Max
          EndIf
        Case #PB_EventType_MouseLeave
          Hover = 0
      EndSelect
      If (Hover <> *RG\Hover)
        *RG\Hover = Hover
        xRatingGadgetRedraw(Gadget, *RG)
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

Procedure.i RatingGadget(Gadget.i, x.i, y.i, Width.i, Height.i, Max.i, OnImageID.i, OffImageID.i = #Null)
  Protected Result.i = #Null
  Protected Canvas.i
  Protected *RG.RATINGGADGET
  
  Canvas = CanvasGadget(Gadget, x, y, Width, Height)
  If (Canvas)
    If (Gadget = #PB_Any)
      Gadget = Canvas
    EndIf
    *RG = AllocateMemory(SizeOf(RATINGGADGET))
    If (*RG)
      InitializeStructure(*RG, RATINGGADGET)
      *RG\State = 0
      *RG\Max   = Max
      *RG\Hover = 0
      *RG\OnImageID  =  OnImageID
      *RG\OffImageID =  OffImageID
      *RG\Background =  xRatingGadgetDefault(#RG_Background)
      *RG\AllowZero  = #True
      SetGadgetData(Gadget, *RG)
      xRatingGadgetRedraw(Gadget, *RG)
      Result = Gadget
    Else
      FreeGadget(Gadget)
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

EndModule


; IDE Options = PureBasic 5.70 LTS (Windows - x86)
; CursorPosition = 269
; FirstLine = 228
; Folding = --
; EnableXP
; CompileSourceDirectory