/*
**
**                           Main.c
**
**
**********************************************************************/
/*
   Last committed:     $Revision: 00 $
   Last changed by:    $Author: $
   Last changed date:  $Date:  $
   ID:                 $Id:  $

**********************************************************************/
#include "stm32f10x_conf.h"
#include "USART_OWire.h"
#include "SysTimer.h"
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_pwr.h"
#include "HidDev_Config.h"
#include "usb.h"
#include "main.h"


#define COUNT_ADC_SAMPLES 10   //    .

#define tV_25   1.36f      //  ( )     25 C.
#define tSlope  0.0043f    //   ( )     .
#define Vref    3.20f       //    ( ).

typedef enum
{
    ow_mode_convert,
    ow_mode_read
} ow_read_mode;

extern uint32_t SystemCoreClock;

volatile sOW_SN OWireDev[COUNT_OW_DEV];
volatile sOW_Temp OWireTemp[COUNT_OW_DEV];
volatile uint16_t CountOWireDev = 0;

volatile uint16_t ADC_DMA_Buff[COUNT_ADC_SAMPLES * 2];
volatile int16_t TempIntSensor=0;


void SearchOWireDev(void) //  1-Wire .
{
    CountOWireDev = 0;
    OW_SearchReset();

    while (OW_SearchEnum((uint8_t*)&OWireDev[CountOWireDev]))
    {
        if (OW_CalcCrc8((uint8_t*)&OWireDev[CountOWireDev], 7) == OWireDev[CountOWireDev].Crc8)
        {
            if ((OWireDev[CountOWireDev].Type == 0x10) || (OWireDev[CountOWireDev].Type == 0x28))
            {
                (*(int16_t*) &OWireTemp[CountOWireDev].L_Termo) = TEMP_NODATA;

                if ((++CountOWireDev)>=COUNT_OW_DEV)
                {
                    break;
                }
            }
        }
    }
}

uint8_t ReadTempOWireDev(ow_read_mode Mode)
{
    uint8_t Res = 0;

   if (CountOWireDev>0)
    {
        if (Mode == ow_mode_convert)
        {
            if (OW_Reset() == OW_OK)
            {
                if (OW_WriteArr((uint8_t*) "\xCC\x44", 2) == OW_OK) //  .
                {
                    Res = 1;
                }
            }
        }
        else if (Mode == ow_mode_read)
        {
            uint16_t i;
            for (i=0; i<CountOWireDev; i++)
            {
                if (OW_Reset() == OW_OK)
                {
                    if ((OW_WriteByte(0x55) == OW_OK) &&
                        (OW_WriteArr((uint8_t*) &OWireDev[i], 8) == OW_OK))
                    {
                        if ((OW_WriteByte(0xBE) == OW_OK) &&
                            (OW_ReadArr((uint8_t*) &OWireTemp[i], 9) == OW_OK))
                        {
                            if (OW_CalcCrc8((uint8_t*) &OWireTemp[i], 8) == OWireTemp[i].CRC8)
                            {
                                Res = 1;
                            }
                            else goto Err;
                        }
                        else goto Err;
                    }
                    else goto Err;
                }
                else
                {
                    Err:;
                    (*(int16_t*) &OWireTemp[i].L_Termo) = TEMP_ERR;
                }
            }
        }
    }

   return Res;
}

void Init_GPIO(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	GPIO_SetBits(GPIOC, GPIO_Pin_13);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

	GPIO_Init(GPIOC, &GPIO_InitStructure);
}

void Init_Adc(void)
{

    RCC_ADCCLKConfig(RCC_PCLK2_Div8);

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); //    3,
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); // ,
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   //  DMA.

    {
        TIM_TimeBaseInitTypeDef Tim;

        Tim.TIM_ClockDivision = TIM_CKD_DIV1; //     .
        Tim.TIM_CounterMode   = TIM_CounterMode_Up; //     .
        Tim.TIM_Prescaler     = (SystemCoreClock / 10000) - 1; //       10 .
        Tim.TIM_Period        = 10000/20-1; //     20   .
        Tim.TIM_RepetitionCounter = 0;

        TIM_TimeBaseInit(TIM3, &Tim); //  .
        TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update); //    .
    }

     //  DMA.
   {
        DMA_InitTypeDef Dma;

        Dma.DMA_PeripheralBaseAddr = (uint32_t) &(ADC1->DR);
        Dma.DMA_MemoryBaseAddr = (uint32_t) ADC_DMA_Buff;
        Dma.DMA_DIR = DMA_DIR_PeripheralSRC;
        Dma.DMA_BufferSize = COUNT_ADC_SAMPLES * 2;
        Dma.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
        Dma.DMA_MemoryInc = DMA_MemoryInc_Enable;
        Dma.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
        Dma.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
        Dma.DMA_Mode = DMA_Mode_Circular;
        Dma.DMA_Priority = DMA_Priority_Low;
        Dma.DMA_M2M = DMA_M2M_Disable;
        DMA_Init(DMA1_Channel1, &Dma); //   .
   }



    DMA_Cmd(DMA1_Channel1 , ENABLE);


    {
        ADC_InitTypeDef Adc; //  .

        Adc.ADC_Mode = ADC_Mode_Independent;
        Adc.ADC_ContinuousConvMode = DISABLE;
        Adc.ADC_ScanConvMode = ENABLE;
        Adc.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T3_TRGO; //     3.
        Adc.ADC_DataAlign = ADC_DataAlign_Right;
        Adc.ADC_NbrOfChannel = 1;
        ADC_Init(ADC1, &Adc);

        //   -   .
        ADC_TempSensorVrefintCmd(ENABLE);

        ADC_RegularChannelConfig(ADC1, ADC_Channel_TempSensor, 1, ADC_SampleTime_239Cycles5);

        ADC_ExternalTrigConvCmd(ADC1, ENABLE);

        ADC_Cmd(ADC1, ENABLE);

        // .
        ADC_ResetCalibration(ADC1);
        while(ADC_GetResetCalibrationStatus(ADC1));
        ADC_StartCalibration(ADC1);
        while(ADC_GetCalibrationStatus(ADC1));

    }

    ADC_DMACmd(ADC1, ENABLE);

    DMA_ITConfig(DMA1_Channel1, DMA_IT_TC | DMA_IT_HT, ENABLE);

    {
        NVIC_InitTypeDef Nvic;
        /* Enable the USARTx Interrupt */
        Nvic.NVIC_IRQChannel = DMA1_Channel1_IRQn;
        Nvic.NVIC_IRQChannelPreemptionPriority = 15;
        Nvic.NVIC_IRQChannelSubPriority = 0;
        Nvic.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&Nvic);
    }

    TIM_Cmd(TIM3, ENABLE); //  .
}


int main(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4); //   0-15.  .

    SystemCoreClockUpdate();
    SysTim_Init(100);

    Set_System();
    USB_Interrupts_Config();
    Set_USBClock();
    USB_Init();

    Init_GPIO();
    Init_Adc();

    OW_Init();

    GPIO_ResetBits(GPIOC, GPIO_Pin_13);
    SearchOWireDev();
    GPIO_SetBits(GPIOC, GPIO_Pin_13);

    ReadTempOWireDev(ow_mode_convert);

    while(1)
    {
        SysTim_DelayMS(800);

        GPIO_ResetBits(GPIOC, GPIO_Pin_13);
        ReadTempOWireDev(ow_mode_read);
        ReadTempOWireDev(ow_mode_convert);
        GPIO_SetBits(GPIOC, GPIO_Pin_13);
    }
}


int16_t SumDataArr(uint16_t *Buff, uint8_t Len) //    .
{
    uint8_t i;
    int32_t Res=0;

    for (i=0; i<Len; i++)
    {
        Res += Buff[i];
    }

    return (int16_t) (Res / Len);
}

int16_t TempCalc(uint16_t AdcRes)
{
    float temp;

    temp = AdcRes/4096.0f*Vref;   //     .
    temp = (tV_25-temp)/tSlope + 25;   //   .

    return (int16_t)(temp*10);
}

void DMA1_Channel1_IRQHandler(void) //     1 DMA1.
{
    uint16_t AdcRes=0;

    if (DMA_GetITStatus(DMA1_IT_HT1)) //   .
    {
        DMA_ClearITPendingBit(DMA1_IT_HT1);
        AdcRes = SumDataArr((uint16_t*) &ADC_DMA_Buff[0], COUNT_ADC_SAMPLES);
    }

    if (DMA_GetITStatus(DMA1_IT_TC1)) //   .
    {
        DMA_ClearITPendingBit(DMA1_IT_TC1);
        AdcRes = SumDataArr((uint16_t*) &ADC_DMA_Buff[COUNT_ADC_SAMPLES], COUNT_ADC_SAMPLES);
    }

    TempIntSensor=TempCalc(AdcRes);
}
